@@ -143,10 +143,10 @@ _maix_display_module = Pybind11Extension(
 #     # extra_compile_args=['-std=c++11', '-std=gnu++11', '-DCONFIG_ARCH_V831'],
 #     extra_link_args=["-Wl,-rpath=/usr/lib/python3.8/site-packages/maix"]
 # )
-max_nn_srcs = get_srcs('ext_modules/_maix_nn_mdsc/src')
+max_nn_srcs = get_srcs('ext_modules/_maix_nn_mud/src')
 max_nn_srcs.extend(get_srcs('ext_modules/libmaix/components/libmaix/src'))
 max_nn_srcs.remove("ext_modules/libmaix/components/libmaix/src/libmaix.c")
-_maix_nn_module = Extension('_maix_nn', include_dirs=['ext_modules/_maix_nn_mdsc/include', 'ext_modules/libmaix/components/libmaix/include'],
+_maix_nn_module = Extension('_maix_nn', include_dirs=['ext_modules/_maix_nn_mud/include', 'ext_modules/libmaix/components/libmaix/include'],
                             sources=max_nn_srcs,
                             libraries=[
     "maix_utils", "maix_nn", "maix_cam",
@@ -170,6 +170,34 @@ _maix_nn_decode_retinaface_module =Pybind11Extension(
     extra_link_args=["-Wl,-rpath=/usr/lib/python3.8/site-packages/maix"]
 )
 
+
+_maix_nn_decode_license_plate_location_module =Pybind11Extension(
+    name = '_maix_nn_decoder_license_plate_location',
+    include_dirs=['ext_modules/libmaix/components/libmaix/include'],
+    sources=['ext_modules/_maix_nn_decoder/py_maix_nn_decoder_license_plate_location.cpp','ext_modules/libmaix/components/libmaix/src/decoder/decoder_license_plate_location.c'] ,
+    libraries=[
+        "pthread",
+        "maix_nn",
+    ],
+    library_dirs=[ ext_so, ],
+    # extra_compile_args=['-std=c++11', '-std=gnu++11', '-DCONFIG_ARCH_V831'],
+    extra_link_args=["-Wl,-rpath=/usr/lib/python3.8/site-packages/maix"]
+)
+
+
+_maix_nn_decode_CTC_module =Pybind11Extension(
+    name = '_maix_nn_decoder_CTC',
+    include_dirs=['ext_modules/libmaix/components/libmaix/include'],
+    sources=['ext_modules/_maix_nn_decoder/py_maix_nn_decoder_ctc.cpp','ext_modules/libmaix/components/libmaix/src/decoder/decoder_ctc.c'] ,
+    libraries=[
+        "pthread",
+        "maix_nn",
+    ],
+    library_dirs=[ ext_so, ],
+    # extra_compile_args=['-std=c++11', '-std=gnu++11', '-DCONFIG_ARCH_V831'],
+    extra_link_args=["-Wl,-rpath=/usr/lib/python3.8/site-packages/maix"]
+)
+
 # _maix_nn_decode_yolo2_module =Pybind11Extension(
 #     name = '_maix_nn_decoder_yolo',
 #     include_dirs=['ext_modules/libmaix/components/libmaix/include'],
@@ -234,7 +262,8 @@ _maix_modules = [
     _maix_display_module,
     _maix_nn_module,
     _maix_nn_decode_retinaface_module,
-
+    _maix_nn_decode_license_plate_location_module,
+    _maix_nn_decode_CTC_module,
 ]
 
 _maix_data_files = [
