@@ -119,6 +119,7 @@ PYBIND11_MODULE(_maix_image, mo)
       .def("get_pixel", &maix_image::_get_pixel, py::arg("x"), py::arg("y"))
       .def("set_pixel", &maix_image::_set_pixel, py::arg("x"), py::arg("y"), py::arg("color"))
       .def("Canny", &maix_image::_opencv_Canny, py::arg("threshold1") = 10, py::arg("threshold2") = 100, py::arg("apertureSize") = 3, py::arg("L2gradient") = false)
+      .def("crop_affine",&maix_image::_warp_affine, py::arg("points"), py::arg("w"), py::arg("h"))
 
       // maix_image继承maix_vision方法
       .def("get_blob_lab", &maix_image::get_blob_color_max, py::arg("roi") = std::vector<int>{0, 0, 0, 0}, py::arg("critical") = 0, py::arg("co") = 0)
@@ -156,5 +157,5 @@ PYBIND11_MODULE(_maix_image, mo)
   pybind11::class_<cv_orb>(mo, "orb")
       .def(pybind11::init<int, float, int, int, int, int, int, int, int>(),
            py::arg("nfeatures") = 500, py::arg("scaleFactor") = 1.2f, py::arg("nlevels") = 8, py::arg("edgeThreshold") = 31, py::arg("firstLevel") = 0, py::arg("WTA_K") = 2, py::arg("scoreType") = 0, py::arg("patchSize") = 31, py::arg("fastThreshold") = 20)
-      .def("match", &cv_orb::match, pybind11::arg("src"), pybind11::arg("dst"), pybind11::arg("limit") = 30, pybind11::arg("max") = 0.5f, pybind11::arg("max") = 100.f, pybind11::arg("crossCheck") = false, pybind11::arg("dump") = 0);
+      .def("match", &cv_orb::match, pybind11::arg("src"), pybind11::arg("dst"), pybind11::arg("limit") = 30, pybind11::arg("min") = 0.0f, pybind11::arg("max") = 100.f, pybind11::arg("crossCheck") = false, pybind11::arg("dump") = 0);
 }
